@@ -27,6 +27,7 @@ from relay.blockchain.delegate import (
     InvalidTimeLimit,
     UnknownIdentityFactoryException,
 )
+from relay.blockchain.exchange_events import all_event_types as all_exchange_event_types
 from relay.blockchain.unw_eth_events import all_event_types as all_unw_eth_event_types
 from relay.ethindex_db.events_informations import (
     EventNotFoundException,
@@ -34,7 +35,7 @@ from relay.ethindex_db.events_informations import (
     TransferNotFoundException,
 )
 from relay.network_graph.payment_path import FeePayer, PaymentPath
-from relay.relay import TrustlinesRelay
+from relay.relay import TrustlinesRelay, all_event_contract_types
 from relay.utils import get_version, sha3
 
 from .schemas import (
@@ -44,7 +45,9 @@ from .schemas import (
     AppliedDelegationFeeSchema,
     CurrencyNetworkEventSchema,
     CurrencyNetworkSchema,
+    DebtsListInCurrencyNetworkSchema,
     IdentityInfosSchema,
+    MediationFeesListSchema,
     MetaTransactionFeeSchema,
     MetaTransactionSchema,
     MetaTransactionStatusSchema,
@@ -53,6 +56,7 @@ from .schemas import (
     TransactionStatusSchema,
     TransferIdentifierSchema,
     TransferInformationSchema,
+    TransferredSumSchema,
     TrustlineSchema,
     TxInfosSchema,
     UserCurrencyNetworkEventSchema,
@@ -379,10 +383,17 @@ class UserEvents(Resource):
         "type": fields.Str(
             required=False,
             validate=validate.OneOf(
-                all_currency_network_event_types + all_unw_eth_event_types
+                all_currency_network_event_types
+                + all_unw_eth_event_types
+                + all_exchange_event_types
             ),
             missing=None,
         ),
+        "contractType": fields.Str(
+            required=False,
+            missing=None,
+            validate=validate.OneOf(all_event_contract_types),
+        ),
     }
 
     @use_args(args)
@@ -390,9 +401,13 @@ class UserEvents(Resource):
     def get(self, args, user_address: str):
         type = args["type"]
         from_block = args["fromBlock"]
+        contract_type = args["contractType"]
 
         return self.trustlines.get_user_events(
-            user_address, type=type, from_block=from_block,
+            user_address,
+            event_type=type,
+            from_block=from_block,
+            contract_type=contract_type,
         )
 
 
@@ -442,15 +457,17 @@ class UserAccruedInterestList(Resource):
         for friend in self.trustlines.get_friends_of_user_in_network(
             network_address, user_address
         ):
-            accrued_interest_list.append(
-                {
-                    "accruedInterests": self.trustlines.get_list_of_accrued_interests_for_trustline(
-                        network_address, user_address, friend, start_time, end_time
-                    ),
-                    "user": user_address,
-                    "counterparty": friend,
-                }
+            accrued_interest_with_friend = self.trustlines.get_list_of_accrued_interests_for_trustline(
+                network_address, user_address, friend, start_time, end_time
             )
+            if len(accrued_interest_with_friend) != 0:
+                accrued_interest_list.append(
+                    {
+                        "accruedInterests": accrued_interest_with_friend,
+                        "user": user_address,
+                        "counterparty": friend,
+                    }
+                )
         return accrued_interest_list
 
 
@@ -480,6 +497,72 @@ class TrustlineAccruedInterestList(Resource):
         }
 
 
+class UserEarnedMediationFeesList(Resource):
+    def __init__(self, trustlines: TrustlinesRelay) -> None:
+        self.trustlines = trustlines
+
+    args = {
+        "startTime": fields.Int(required=False, missing=0),
+        "endTime": fields.Int(required=False, missing=None),
+    }
+
+    @use_args(args)
+    @dump_result_with_schema(MediationFeesListSchema())
+    def get(self, args, network_address: str, user_address):
+        abort_if_unknown_network(self.trustlines, network_address)
+        start_time = args["startTime"]
+        end_time = args["endTime"]
+        mediationFees = self.trustlines.get_earned_mediation_fees(
+            network_address, user_address, start_time, end_time
+        )
+
+        return {
+            "mediationFees": mediationFees,
+            "user": user_address,
+            "network": network_address,
+        }
+
+
+class UserDebtsLists(Resource):
+    def __init__(self, trustlines: TrustlinesRelay) -> None:
+        self.trustlines = trustlines
+
+    @dump_result_with_schema(DebtsListInCurrencyNetworkSchema(many=True))
+    def get(self, user_address):
+        return self.trustlines.get_debt_list_of_user(user_address)
+
+
+class TotalTransferredSum(Resource):
+    def __init__(self, trustlines: TrustlinesRelay) -> None:
+        self.trustlines = trustlines
+
+    args = {
+        "startTime": fields.Int(required=False, missing=0),
+        "endTime": fields.Int(required=False, missing=None),
+    }
+
+    @use_args(args)
+    @dump_result_with_schema(TransferredSumSchema())
+    def get(self, args, network_address: str, sender_address, receiver_address):
+        abort_if_unknown_network(self.trustlines, network_address)
+        start_time = args["startTime"]
+        end_time = args["endTime"]
+
+        if end_time is None:
+            end_time = time.time()
+
+        value = self.trustlines.get_total_sum_transferred(
+            network_address, sender_address, receiver_address, start_time, end_time
+        )
+        return {
+            "sender": sender_address,
+            "receiver": receiver_address,
+            "startTime": start_time,
+            "endTime": end_time,
+            "value": value,
+        }
+
+
 class TransferInformation(Resource):
     def __init__(self, trustlines: TrustlinesRelay) -> None:
         self.trustlines = trustlines
