@@ -1,6 +1,7 @@
 import json
 import logging
 import os
+import sys
 from collections import defaultdict
 from copy import deepcopy
 from enum import Enum
@@ -12,7 +13,7 @@ import gevent
 import sqlalchemy
 from eth_utils import is_checksum_address, to_checksum_address
 from sqlalchemy.engine.url import URL
-from tlbin import load_packaged_contracts
+from tlbin import load_packaged_contracts, load_packaged_merged_abis
 from tldeploy.identity import MetaTransaction
 from web3 import Web3
 
@@ -434,7 +435,8 @@ class TrustlinesRelay:
                 self._publish_feed_update_events(graph_updates)
                 gevent.sleep(self.config["trustline_index"]["sync_interval"])
 
-        gevent.Greenlet.spawn(sync)
+        greenlet = gevent.Greenlet.spawn(sync)
+        greenlet.link_exception(lambda *args: sys.exit("Graph sync greenlet died"))
 
     def new_network(self, address: str) -> None:
         assert is_checksum_address(address)
@@ -442,7 +444,9 @@ class TrustlinesRelay:
             return
         logger.info("New network: {}".format(address))
         self.currency_network_proxies[address] = CurrencyNetworkProxy(
-            self._web3, self.contracts["CurrencyNetwork"]["abi"], address
+            self._web3,
+            load_packaged_merged_abis()["MergedCurrencyNetworksAbi"]["abi"],
+            address,
         )
         currency_network_proxy = self.currency_network_proxies[address]
         self.currency_network_graphs[address] = CurrencyNetworkGraph(
@@ -763,30 +767,49 @@ class TrustlinesRelay:
     def _publish_feed_update_events(
         self, feed_update: Iterable[FeedUpdate],
     ):
-        def unique_updated_id(from_, to, network_address):
+        # We want to publish only the latest update for each user / trustline
+        feed_update = sorted(
+            feed_update, key=lambda update: update.timestamp, reverse=True
+        )
+
+        def unique_trustline_id(from_, to, network_address):
             if from_ > to:
                 return from_ + to + network_address
             else:
                 return to + from_ + network_address
 
-        processed_updates: Set[str] = set()
+        processed_trusline_updates: Set[str] = set()
+        processed_user_updates: Set[str] = set()
 
         for update in feed_update:
             if update.address not in self.currency_network_graphs.keys():
                 continue
+
             if type(update) in [TrustlineUpdateFeedUpdate, BalanceUpdateFeedUpdate]:
                 update = cast(
                     Union[TrustlineUpdateFeedUpdate, BalanceUpdateFeedUpdate], update
                 )
-                uid = unique_updated_id(update.from_, update.to, update.address)
-                if uid not in processed_updates:
+
+                trustline_id = unique_trustline_id(
+                    update.from_, update.to, update.address
+                )
+                if trustline_id not in processed_trusline_updates:
                     self._publish_trustline_events(
                         user1=update.from_,
                         user2=update.to,
                         network_address=update.address,
                         timestamp=update.timestamp,
                     )
-                processed_updates.add(uid)
+                    processed_trusline_updates.add(trustline_id)
+
+                for user in [update.from_, update.to]:
+                    if user not in processed_user_updates:
+                        self._publish_network_balance_event(
+                            user=user,
+                            network_address=update.address,
+                            timestamp=update.timestamp,
+                        )
+                        processed_user_updates.add(user)
 
     def _load_gas_price_settings(self, gas_price_settings: Dict):
         method = gas_price_settings["method"]
@@ -909,17 +932,17 @@ class TrustlinesRelay:
                     timestamp,
                 )
             )
-        for user in [user1, user2]:
-            events.append(
-                NetworkBalanceEvent(
-                    network_address,
-                    user,
-                    graph.get_account_sum(user, timestamp=timestamp),
-                    timestamp,
-                )
-            )
         return events
 
+    def _generate_network_balance_event(self, *, user, network_address, timestamp):
+        graph = self.currency_network_graphs[network_address]
+        return NetworkBalanceEvent(
+            network_address,
+            user,
+            graph.get_account_sum(user, timestamp=timestamp),
+            timestamp,
+        )
+
     def _publish_trustline_events(self, *, user1, user2, network_address, timestamp):
         events = self._generate_trustline_events(
             user1=user1,
@@ -930,6 +953,12 @@ class TrustlinesRelay:
         for ev in events:
             self._publish_user_event(ev)
 
+    def _publish_network_balance_event(self, *, user, network_address, timestamp):
+        event = self._generate_network_balance_event(
+            user=user, network_address=network_address, timestamp=timestamp
+        )
+        self._publish_user_event(event)
+
     def _process_trustline_update(self, trustline_update_event):
         logger.debug("Process trustline update event: %s", trustline_update_event)
         self._publish_blockchain_event(trustline_update_event)
